@@ -1,8 +1,8 @@
 import os.path
 import sys
 
-sys.path.append(os.path.abspath(".."))
-sys.path.append(os.path.abspath("exts"))
+sys.path.insert(0, os.path.abspath(".."))
+sys.path.insert(0, os.path.abspath("exts"))
 
 master_doc = "index"
 
@@ -26,6 +26,10 @@ drgndoc_paths = ["../drgn", "../_drgn.pyi"]
 drgndoc_substitutions = [
     (r"^_drgn\b", "drgn"),
 ]
+drgndoc_submodule_sort = [
+    # Sort experimental helpers after everything else.
+    (r"drgn\.helpers", [(r"experimental", 1)]),
+]
 
 extlinks = {
     "contrib": (
