@@ -2,6 +2,7 @@
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
 import ast
+import dataclasses
 import inspect
 import operator
 import os.path
@@ -59,54 +60,26 @@ class _PreTransformer(ast.NodeTransformer):
         node.annotation = self._visit_annotation(node.annotation)
         return node
 
-    # Replace the old constant nodes produced by ast.parse() before Python 3.8
-    # with Constant.
-    def visit_Num(self, node: Any) -> ast.Constant:
-        return ast.copy_location(ast.Constant(node.n), node)
-
-    def visit_Str(self, node: Any) -> ast.Constant:
-        return ast.copy_location(ast.Constant(node.s), node)
-
-    def visit_Bytes(self, node: Any) -> ast.Constant:
-        return ast.copy_location(ast.Constant(node.s), node)
-
-    def visit_Ellipsis(self, node: Any) -> ast.Constant:
-        return ast.copy_location(ast.Constant(...), node)
-
-    def visit_NameConstant(self, node: Any) -> ast.Constant:
-        return ast.copy_location(ast.Constant(node.value), node)
-
     # Get rid of Index nodes, which are deprecated as of Python 3.9.
     def visit_Index(self, node: Any) -> Any:
         return self.visit(node.value)
 
 
-# Once we don't care about Python 3.6, we can replace all of this boilerplate
-# with dataclasses.
-
-
+@dataclasses.dataclass
 class Module:
-    def __init__(
-        self, path: Optional[str], docstring: Optional[str], attrs: Mapping[str, "Node"]
-    ) -> None:
-        self.path = path
-        self.docstring = docstring
-        self.attrs = attrs
+    path: Optional[str]
+    docstring: Optional[str]
+    attrs: Mapping[str, "Node"]
 
     def has_docstring(self) -> bool:
         return self.docstring is not None
 
 
+@dataclasses.dataclass
 class Class:
-    def __init__(
-        self,
-        bases: Sequence[ast.expr],
-        docstring: Optional[str],
-        attrs: Mapping[str, "NonModuleNode"],
-    ) -> None:
-        self.bases = bases
-        self.docstring = docstring
-        self.attrs = attrs
+    bases: Sequence[ast.expr]
+    docstring: Optional[str]
+    attrs: Mapping[str, "NonModuleNode"]
 
     def has_docstring(self) -> bool:
         if self.docstring is not None:
@@ -115,18 +88,12 @@ class Class:
         return isinstance(init, Function) and init.has_docstring()
 
 
+@dataclasses.dataclass
 class FunctionSignature:
-    def __init__(
-        self,
-        args: ast.arguments,
-        returns: Optional[ast.expr],
-        decorator_list: Sequence[ast.expr],
-        docstring: Optional[str],
-    ) -> None:
-        self.args = args
-        self.returns = returns
-        self.decorator_list = decorator_list
-        self.docstring = docstring
+    args: ast.arguments
+    returns: Optional[ast.expr]
+    decorator_list: Sequence[ast.expr]
+    docstring: Optional[str]
 
     def has_decorator(self, name: str) -> bool:
         return any(
@@ -135,43 +102,40 @@ class FunctionSignature:
         )
 
 
+@dataclasses.dataclass
 class Function:
-    def __init__(self, async_: bool, signatures: Sequence[FunctionSignature]) -> None:
-        self.async_ = async_
-        self.signatures = signatures
+    async_: bool
+    signatures: Sequence[FunctionSignature]
 
     def has_docstring(self) -> bool:
         return any(signature.docstring is not None for signature in self.signatures)
 
 
+@dataclasses.dataclass
 class Variable:
-    def __init__(
-        self, annotation: Optional[ast.expr], docstring: Optional[str]
-    ) -> None:
-        self.annotation = annotation
-        self.docstring = docstring
+    annotation: Optional[ast.expr]
+    value: Optional[ast.expr]
+    docstring: Optional[str]
 
     def has_docstring(self) -> bool:
         return self.docstring is not None
 
 
+@dataclasses.dataclass
 class Import:
-    def __init__(self, module: str, aliased: bool) -> None:
-        self.module = module
-        self.aliased = aliased
+    module: str
+    aliased: bool
 
     def has_docstring(self) -> bool:
         return False
 
 
+@dataclasses.dataclass
 class ImportFrom:
-    def __init__(
-        self, name: str, module: Optional[str], level: int, aliased: bool
-    ) -> None:
-        self.name = name
-        self.module = module
-        self.level = level
-        self.aliased = aliased
+    name: str
+    module: Optional[str]
+    level: int
+    aliased: bool
 
     def has_docstring(self) -> bool:
         return False
@@ -291,8 +255,8 @@ class _ModuleVisitor(NodeVisitor):
     def _add_assign(
         self,
         name: str,
-        have_value: bool,
         annotation: Optional[ast.expr],
+        value: Optional[ast.expr],
         docstring: Optional[str],
     ) -> None:
         try:
@@ -301,19 +265,19 @@ class _ModuleVisitor(NodeVisitor):
             pass
         else:
             # The name was previously defined. If it's a variable, add the
-            # annotation and/or docstring. If this is an annotation without a
-            # value, don't do anything. Otherwise, replace the previous
-            # definition.
+            # annotation, value, and/or docstring. If this is an annotation
+            # without a value, don't do anything. Otherwise, replace the
+            # previous definition.
             if isinstance(var, Variable):
-                if not annotation and docstring is None:
-                    return
-                if not annotation:
+                if annotation is None:
                     annotation = var.annotation
+                if value is None:
+                    value = var.value
                 if docstring is None:
                     docstring = var.docstring
-            elif not have_value:
+            elif value is None:
                 return
-        self._attrs[name] = Variable(annotation, docstring)
+        self._attrs[name] = Variable(annotation, value, docstring)
 
     def visit_Assign(
         self, node: ast.Assign, parent: Optional[ast.AST], sibling: Optional[ast.AST]
@@ -324,7 +288,7 @@ class _ModuleVisitor(NodeVisitor):
             docstring = None
         for target in node.targets:
             if isinstance(target, ast.Name):
-                self._add_assign(target.id, True, None, docstring)
+                self._add_assign(target.id, None, node.value, docstring)
 
     def visit_AnnAssign(
         self, node: ast.AnnAssign, parent: Optional[ast.AST], sibling: Optional[ast.AST]
@@ -332,8 +296,8 @@ class _ModuleVisitor(NodeVisitor):
         if isinstance(node.target, ast.Name):
             self._add_assign(
                 node.target.id,
-                node.value is not None,
                 node.annotation,
+                node.value,
                 _docstring_from_node(sibling),
             )
 
