@@ -18,6 +18,7 @@ __all__ = (
     "per_cpu",
     "per_cpu_ptr",
     "percpu_counter_sum",
+    "percpu_counter_sum_positive",
 )
 
 
@@ -45,7 +46,21 @@ def percpu_counter_sum(fbc: Object) -> int:
     :param fbc: ``struct percpu_counter *``
     """
     ret = fbc.count.value_()
-    ptr = fbc.counters
-    for cpu in for_each_online_cpu(fbc.prog_):
-        ret += per_cpu_ptr(ptr, cpu)[0].value_()
+    try:
+        ptr = fbc.counters
+    except AttributeError:
+        # On !SMP kernels, there's nothing to sum.
+        pass
+    else:
+        for cpu in for_each_online_cpu(fbc.prog_):
+            ret += per_cpu_ptr(ptr, cpu)[0].value_()
     return ret
+
+
+def percpu_counter_sum_positive(fbc: Object) -> int:
+    """
+    Return the sum of a per-CPU counter, or 0 if it is negative.
+
+    :param fbc: ``struct percpu_counter *``
+    """
+    return max(percpu_counter_sum(fbc), 0)
