@@ -43,6 +43,14 @@ struct drgn_module;
  */
 struct drgn_error *read_elf_section(Elf_Scn *scn, Elf_Data **ret);
 
+/**
+ * Truncate any bytes beyond the last null character in an ELF string table.
+ *
+ * This sets `data->d_size` so that any string table index less than
+ * `data->d_size` is guaranteed to be valid.
+ */
+void truncate_elf_string_data(Elf_Data *data);
+
 static inline bool elf_data_contains_ptr(Elf_Data *data, const void *ptr)
 {
 	uintptr_t bufi = (uintptr_t)data->d_buf;
@@ -55,11 +63,38 @@ struct drgn_elf_file {
 	/** Module using this file. */
 	struct drgn_module *module;
 	/** Filesystem path to this file. */
-	const char *path;
+	char *path;
+	/**
+	 * Memory image backing @ref elf.
+	 *
+	 * @c NULL if not backed by a memory image.
+	 */
+	char *image;
+	/**
+	 * File descriptor backing @ref elf.
+	 *
+	 * -1 if not backed by a file.
+	 */
+	int fd;
+	/** Whether the file is loadable. */
+	bool is_loadable;
+	/** Whether the file is relocatable. */
+	bool is_relocatable;
+	/** Whether the file still need to have relocations applied. */
+	bool needs_relocation;
+	/** Whether the file is a Linux kernel image (`vmlinux`). */
+	bool is_vmlinux;
 	/** libelf handle. */
 	Elf *elf;
-	/** libdw handle if we're using DWARF information from this file. */
-	Dwarf *dwarf;
+	/**
+	 * libdw handle.
+	 *
+	 * @c NULL if not yet created.
+	 *
+	 * Don't access this directly. Get it with @ref
+	 * drgn_elf_file_get_dwarf() instead.
+	 */
+	Dwarf *_dwarf;
 	/**
 	 * Platform of this file.
 	 *
@@ -86,16 +121,33 @@ struct drgn_elf_file {
 	Elf_Data *alt_debug_str_data;
 };
 
+/**
+ * Create a @ref drgn_elf_file.
+ *
+ * On success, this takes ownership of @p fd, @p image, and @p elf. @p path is
+ * copied.
+ */
 struct drgn_error *drgn_elf_file_create(struct drgn_module *module,
-					const char *path, Elf *elf,
-					struct drgn_elf_file **ret);
+					const char *path, int fd, char *image,
+					Elf *elf, struct drgn_elf_file **ret);
 
 void drgn_elf_file_destroy(struct drgn_elf_file *file);
 
-struct drgn_error *drgn_elf_file_precache_sections(struct drgn_elf_file *file);
-
+/** Apply ELF relocations to the file if needed. */
 struct drgn_error *
-drgn_elf_file_cache_section(struct drgn_elf_file *file, enum drgn_section_index scn);
+drgn_elf_file_apply_relocations(struct drgn_elf_file *file);
+
+/**
+ * Read an indexed ELF section.
+ *
+ * This applies ELF relocations to the file first if needed.
+ */
+struct drgn_error *drgn_elf_file_read_section(struct drgn_elf_file *file,
+					      enum drgn_section_index scn,
+					      Elf_Data **ret);
+
+struct drgn_error *drgn_elf_file_get_dwarf(struct drgn_elf_file *file,
+					   Dwarf **ret);
 
 static inline bool
 drgn_elf_file_is_little_endian(const struct drgn_elf_file *file)
@@ -108,6 +160,12 @@ static inline bool drgn_elf_file_bswap(const struct drgn_elf_file *file)
 	return drgn_platform_bswap(&file->platform);
 }
 
+static inline bool
+drgn_elf_file_is_64_bit(const struct drgn_elf_file *file)
+{
+	return drgn_platform_is_64_bit(&file->platform);
+}
+
 static inline uint8_t
 drgn_elf_file_address_size(const struct drgn_elf_file *file)
 {
@@ -120,6 +178,12 @@ drgn_elf_file_address_mask(const struct drgn_elf_file *file)
 	return drgn_platform_address_mask(&file->platform);
 }
 
+static inline bool drgn_elf_file_has_dwarf(const struct drgn_elf_file *file)
+{
+	return (file->scns[DRGN_SCN_DEBUG_INFO]
+		&& file->scns[DRGN_SCN_DEBUG_ABBREV]);
+}
+
 struct drgn_error *
 drgn_elf_file_section_error(struct drgn_elf_file *file, Elf_Scn *scn,
 			    Elf_Data *data, const char *ptr,
@@ -156,6 +220,10 @@ drgn_elf_file_section_buffer_init(struct drgn_elf_file_section_buffer *buffer,
 	buffer->data = data;
 }
 
+/**
+ * Initialize a @ref binary_buffer for an indexed ELF section that has already
+ * been read.
+ */
 static inline void
 drgn_elf_file_section_buffer_init_index(struct drgn_elf_file_section_buffer *buffer,
 					struct drgn_elf_file *file,
@@ -165,6 +233,47 @@ drgn_elf_file_section_buffer_init_index(struct drgn_elf_file_section_buffer *buf
 					  file->scn_data[scn]);
 }
 
+/**
+ * Read an indexed ELF section (applying ELF relocations if needed) and
+ * initialize a @ref binary_buffer for it.
+ */
+static inline struct drgn_error *
+drgn_elf_file_section_buffer_read(struct drgn_elf_file_section_buffer *buffer,
+				  struct drgn_elf_file *file,
+				  enum drgn_section_index scn)
+{
+	Elf_Data *data;
+	struct drgn_error *err = drgn_elf_file_read_section(file, scn, &data);
+	if (err)
+		return err;
+	drgn_elf_file_section_buffer_init(buffer, file, file->scns[scn], data);
+	return NULL;
+}
+
+/**
+ * Return the virtual address range of an ELF file.
+ *
+ * @param[out] start_ret Minimum virtual address (inclusive).
+ * @param[out] end_ret Maximum virtual address (exclusive).
+ */
+bool drgn_elf_file_address_range(struct drgn_elf_file *file,
+				 uint64_t *start_ret, uint64_t *end_ret);
+
+/**
+ * Return whether an ELF file is a vmlinux file.
+ *
+ * @return > 0 if the file is vmlinux, 0 if it is not, < 0 on libelf error.
+ */
+int elf_is_vmlinux(Elf *elf);
+
+/**
+ * Get the Linux release from a vmlinux file.
+ *
+ * @param[out] ret Returned release.
+ * @return Length of @p ret on success, 0 if not found, < 0 on libelf error.
+ */
+ssize_t elf_vmlinux_release(Elf *elf, const char **ret);
+
 /** @} */
 
 #endif /* DRGN_ELF_FILE_H */
