@@ -48,19 +48,23 @@ enum drgn_language_number {
 
 typedef struct drgn_error *drgn_format_type_fn(struct drgn_qualified_type,
 					       char **);
-typedef struct drgn_error *drgn_format_object_fn(const struct drgn_object *,
-						 size_t,
-						 enum drgn_format_object_flags,
-						 char **);
+typedef struct drgn_error *
+drgn_format_variable_declaration_fn(struct drgn_qualified_type, const char *,
+				    char **);
+typedef struct drgn_error *
+drgn_format_object_fn(const struct drgn_object *,
+		      const struct drgn_format_object_options *,
+		      char **);
 typedef struct drgn_error *drgn_find_type_fn(const struct drgn_language *lang,
 					     struct drgn_program *prog,
 					     const char *name,
 					     const char *filename,
 					     struct drgn_qualified_type *ret);
-typedef struct drgn_error *drgn_bit_offset_fn(struct drgn_program *prog,
-					      struct drgn_type *type,
-					      const char *member_designator,
-					      uint64_t *ret);
+typedef struct drgn_error *
+drgn_type_subobject_fn(struct drgn_type *type, const char *designator,
+		       bool expect_member,
+		       struct drgn_qualified_type *type_ret,
+		       uint64_t *bit_offset_ret, uint64_t *bit_field_size_ret);
 typedef struct drgn_error *drgn_integer_literal_fn(struct drgn_object *res,
 						   uint64_t uvalue);
 typedef struct drgn_error *drgn_bool_literal_fn(struct drgn_object *res,
@@ -70,6 +74,11 @@ typedef struct drgn_error *drgn_float_literal_fn(struct drgn_object *res,
 typedef struct drgn_error *
 drgn_cast_op(struct drgn_object *res, struct drgn_qualified_type qualified_type,
 	     const struct drgn_object *obj);
+typedef struct drgn_error *
+drgn_implicit_convert_op(struct drgn_object *res,
+			 struct drgn_qualified_type qualified_type,
+			 uint64_t bit_field_size,
+			 const struct drgn_object *obj);
 typedef struct drgn_error *drgn_bool_op(const struct drgn_object *obj, bool *ret);
 typedef struct drgn_error *drgn_cmp_op(const struct drgn_object *lhs,
 				       const struct drgn_object *rhs, int *ret);
@@ -94,6 +103,8 @@ struct drgn_language {
 	drgn_format_type_fn *format_type_name;
 	/** Implement @ref drgn_format_type(). */
 	drgn_format_type_fn *format_type;
+	/** Implement @ref drgn_format_variable_declaration(). */
+	drgn_format_variable_declaration_fn *format_variable_declaration;
 	/** Implement @ref drgn_format_object(). */
 	drgn_format_object_fn *format_object;
 	/**
@@ -104,13 +115,19 @@ struct drgn_language {
 	 */
 	drgn_find_type_fn *find_type;
 	/**
-	 * Get the offset of a member in a type.
+	 * Get the type, offset, and bit field size of a subobject of a type.
 	 *
-	 * This should parse @p member_designator (which may include one or more
-	 * member references and zero or more array subscripts) and calculate
-	 * the offset, in bits, of that member from the beginning of @p type.
+	 * @param[in] type Starting type.
+	 * @param[in] designator One or more member references or array
+	 * subscripts.
+	 * @param[in] expect_member Require a member reference first.
+	 * @param[out] type_ret If not @c NULL, returned subobject type.
+	 * @param[out] bit_offset_ret If not @c NULL, returned offset in bits of
+	 * subobject from the beginning of @p type.
+	 * @param[out] bit_field_size_ret If not @c NULL, returned bit field
+	 * size of subobject.
 	 */
-	drgn_bit_offset_fn *bit_offset;
+	drgn_type_subobject_fn *type_subobject;
 	/**
 	 * Set an object to an integer literal.
 	 *
@@ -133,7 +150,7 @@ struct drgn_language {
 	 */
 	drgn_float_literal_fn *float_literal;
 	drgn_cast_op *op_cast;
-	drgn_cast_op *op_implicit_convert;
+	drgn_implicit_convert_op *op_implicit_convert;
 	drgn_bool_op *op_bool;
 	drgn_cmp_op *op_cmp;
 	drgn_binary_op *op_add;
