@@ -385,22 +385,24 @@ drgn_object_set_from_buffer_internal(struct drgn_object *res,
 				return &drgn_enomem;
 			value.bufp = dst;
 		}
-		int dst_bit_offset = 0;
-		if (type->encoding != DRGN_OBJECT_ENCODING_BUFFER
-		    && !type->little_endian)
-			dst_bit_offset = -type->bit_size % 8;
-		((uint8_t *)dst)[0] = 0;
-		((uint8_t *)dst)[size - 1] = 0;
-		copy_bits(dst, dst_bit_offset, p, bit_offset, type->bit_size,
-			  type->little_endian);
-		if (type->encoding == DRGN_OBJECT_ENCODING_SIGNED_BIG
-		    && type->bit_size % 8 != 0) {
-			int8_t *dst_p;
-			if (type->little_endian)
-				dst_p = (int8_t *)dst + size - 1;
-			else
-				dst_p = (int8_t *)dst;
-			*dst_p = truncate_signed8(*dst_p, type->bit_size % 8);
+		if (size > 0) {
+			int dst_bit_offset = 0;
+			if (type->encoding != DRGN_OBJECT_ENCODING_BUFFER
+			    && !type->little_endian)
+				dst_bit_offset = -type->bit_size % 8;
+			((uint8_t *)dst)[0] = 0;
+			((uint8_t *)dst)[size - 1] = 0;
+			copy_bits(dst, dst_bit_offset, p, bit_offset,
+				  type->bit_size, type->little_endian);
+			if (type->encoding == DRGN_OBJECT_ENCODING_SIGNED_BIG
+			    && type->bit_size % 8 != 0) {
+				int8_t *dst_p;
+				if (type->little_endian)
+					dst_p = (int8_t *)dst + size - 1;
+				else
+					dst_p = (int8_t *)dst;
+				*dst_p = truncate_signed8(*dst_p, type->bit_size % 8);
+			}
 		}
 	} else if (drgn_object_encoding_is_complete(type->encoding)) {
 		if (type->encoding == DRGN_OBJECT_ENCODING_FLOAT
@@ -570,14 +572,16 @@ struct drgn_error *
 drgn_object_fragment_internal(struct drgn_object *res,
 			      const struct drgn_object *obj,
 			      const struct drgn_object_type *type,
-			      uint64_t bit_offset, uint64_t bit_field_size)
+			      int64_t bit_offset, uint64_t bit_field_size)
 {
 	struct drgn_error *err;
 
 	SWITCH_ENUM(obj->kind) {
 	case DRGN_OBJECT_VALUE: {
 		uint64_t bit_end;
-		if (__builtin_add_overflow(bit_offset, type->bit_size, &bit_end)
+		if (bit_offset < 0
+		    || __builtin_add_overflow(bit_offset, type->bit_size,
+					      &bit_end)
 		    || bit_end > obj->bit_size) {
 			return drgn_error_create(DRGN_ERROR_OUT_OF_BOUNDS,
 						 "out of bounds of value");
@@ -603,14 +607,19 @@ drgn_object_fragment_internal(struct drgn_object *res,
 		return drgn_object_set_from_buffer_internal(res, type, buf,
 							    bit_offset);
 	}
-	case DRGN_OBJECT_REFERENCE:
+	case DRGN_OBJECT_REFERENCE: {
 		// obj->bit_offset + bit_offset can overflow, so apply the
 		// byte-aligned part of bit_offset now.
-		return drgn_object_set_reference_internal(res, type,
-							  obj->address
-							  + (bit_offset / 8),
+		//
+		// / and % truncate towards 0. Here, we want to truncate towards
+		// negative infinity. We can accomplish that by replacing "/ 8"
+		// with an arithmetic shift ">> 3" and "% 8" with "& 7".
+		uint64_t address = obj->address + (bit_offset >> 3);
+		bit_offset &= 7;
+		return drgn_object_set_reference_internal(res, type, address,
 							  obj->bit_offset
-							  + (bit_offset % 8));
+							  + bit_offset);
+	}
 	case DRGN_OBJECT_ABSENT:
 		return &drgn_error_object_absent;
 	default:
@@ -621,7 +630,7 @@ drgn_object_fragment_internal(struct drgn_object *res,
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_object_fragment(struct drgn_object *res, const struct drgn_object *obj,
 		     struct drgn_qualified_type qualified_type,
-		     uint64_t bit_offset, uint64_t bit_field_size)
+		     int64_t bit_offset, uint64_t bit_field_size)
 {
 	struct drgn_error *err;
 	if (drgn_object_program(res) != drgn_object_program(obj)) {
@@ -1099,16 +1108,33 @@ drgn_object_read_c_string(const struct drgn_object *obj, char **ret)
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
-drgn_format_object(const struct drgn_object *obj, size_t columns,
-		   enum drgn_format_object_flags flags, char **ret)
+drgn_format_object(const struct drgn_object *obj,
+		   const struct drgn_format_object_options *options,
+		   char **ret)
 {
+
 	const struct drgn_language *lang = drgn_object_language(obj);
 
-	if (flags & ~DRGN_FORMAT_OBJECT_VALID_FLAGS) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "invalid format object flags");
+	if (options) {
+		if (options->flags & ~DRGN_FORMAT_OBJECT_VALID_FLAGS) {
+			return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						 "invalid format object flags");
+		}
+		if (options->integer_base != 8
+		    && options->integer_base != 10
+		    && options->integer_base != 16) {
+			return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+						 "invalid integer base");
+		}
+	} else {
+		static const struct drgn_format_object_options default_options = {
+			.columns = SIZE_MAX,
+			.flags = DRGN_FORMAT_OBJECT_PRETTY,
+			.integer_base = 10,
+		};
+		options = &default_options;
 	}
-	return lang->format_object(obj, columns, flags, ret);
+	return lang->format_object(obj, options, ret);
 }
 
 static struct drgn_error *
@@ -1369,6 +1395,7 @@ drgn_object_cast(struct drgn_object *res,
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_object_implicit_convert(struct drgn_object *res,
 			     struct drgn_qualified_type qualified_type,
+			     uint64_t bit_field_size,
 			     const struct drgn_object *obj)
 {
 	const struct drgn_language *lang = drgn_type_language(qualified_type.type);
@@ -1377,7 +1404,8 @@ drgn_object_implicit_convert(struct drgn_object *res,
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "objects are from different programs");
 	}
-	return lang->op_implicit_convert(res, qualified_type, obj);
+	return lang->op_implicit_convert(res, qualified_type, bit_field_size,
+					 obj);
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
@@ -1547,7 +1575,7 @@ drgn_object_address_of(struct drgn_object *res, const struct drgn_object *obj)
 
 	struct drgn_qualified_type qualified_type =
 		drgn_object_qualified_type(obj);
-	uint8_t address_size;
+	uint64_t address_size;
 	struct drgn_error *err =
 		drgn_program_address_size(drgn_object_program(obj),
 					  &address_size);
@@ -1593,6 +1621,44 @@ drgn_object_subscript(struct drgn_object *res, const struct drgn_object *obj,
 	}
 }
 
+LIBDRGN_PUBLIC
+struct drgn_error *drgn_object_slice(struct drgn_object *res,
+				     const struct drgn_object *obj,
+				     int64_t start, int64_t end)
+{
+	struct drgn_error *err;
+	struct drgn_program *prog = drgn_object_program(obj);
+
+	if (drgn_object_program(res) != prog) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "objects are from different programs");
+	}
+
+	struct drgn_element_info element;
+	err = drgn_program_element_info(drgn_object_program(obj), obj->type,
+					&element);
+	if (err)
+		return err;
+
+	struct drgn_qualified_type array_type;
+	err = drgn_array_type_create(prog, element.qualified_type,
+				     end > start ? end - start : 0,
+				     drgn_type_language(element.qualified_type.type),
+				     &array_type.type);
+	if (err)
+		return err;
+	array_type.qualifiers = 0;
+
+	if (obj->encoding == DRGN_OBJECT_ENCODING_UNSIGNED) {
+		return drgn_object_dereference_offset(res, obj, array_type,
+						      start * element.bit_size,
+						      0);
+	} else {
+		return drgn_object_fragment(res, obj, array_type,
+					    start * element.bit_size, 0);
+	}
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_object_member(struct drgn_object *res, const struct drgn_object *obj,
 		   const char *member_name)
@@ -1653,6 +1719,24 @@ drgn_object_member_dereference(struct drgn_object *res,
 					      member_bit_field_size);
 }
 
+LIBDRGN_PUBLIC
+struct drgn_error *drgn_object_subobject(struct drgn_object *res,
+					 const struct drgn_object *obj,
+					 const char *designator)
+{
+	struct drgn_error *err;
+	const struct drgn_language *lang = drgn_object_language(obj);
+	struct drgn_qualified_type qualified_type;
+	uint64_t bit_offset, bit_field_size;
+	err = lang->type_subobject(obj->type, designator, false,
+				   &qualified_type, &bit_offset,
+				   &bit_field_size);
+	if (err)
+		return err;
+	return drgn_object_fragment(res, obj, qualified_type, bit_offset,
+				    bit_field_size);
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_object_container_of(struct drgn_object *res, const struct drgn_object *obj,
 			 struct drgn_qualified_type qualified_type,
@@ -1682,7 +1766,7 @@ drgn_object_container_of(struct drgn_object *res, const struct drgn_object *obj,
 	if (err)
 		return err;
 
-	uint8_t address_size;
+	uint64_t address_size;
 	err = drgn_program_address_size(drgn_object_program(obj),
 					&address_size);
 	if (err)
