@@ -71,6 +71,8 @@ struct drgn_program {
 	Elf *core;
 	/* File descriptor for ELF core dump, kdump file, or /proc/pid/mem. */
 	int core_fd;
+	/* Path of core dump. */
+	char *core_path;
 	/* PID of live userspace program. */
 	pid_t pid;
 #ifdef WITH_LIBKDUMPFILE
@@ -140,6 +142,12 @@ struct drgn_program {
 	struct drgn_thread_set thread_set;
 	struct drgn_thread *main_thread;
 	struct drgn_thread *crashed_thread;
+	/*
+	 * Cached offsetof(struct pt_regs, stackframe) for aarch64, or 0 if not
+	 * yet cached.
+	 */
+	uint64_t aarch64_stackframe_offset_cached;
+
 	/*
 	 * AArch64 instruction pointer authentication code mask, parsed either
 	 * from NT_ARM_PAC_MASK or VMCOREINFO.
@@ -193,6 +201,18 @@ struct drgn_program {
 				 * `NR_SECTION_ROOTS`).
 				 */
 				uint64_t mem_section_length;
+				/**
+				 * `SECTION_SIZE_BITS` of the kernel. Initially
+				 * 0 if not found in VMCOREINFO, but may be
+				 * determined by other means and cached later.
+				 */
+				int section_size_bits;
+				/**
+				 * `MAX_PHYSMEM_BITS` of the kernel. Initially 0
+				 * if not found in VMCOREINFO, but may be
+				 * determined by other means and cached later.
+				 */
+				int max_physmem_bits;
 				/** `VA_BITS` on AArch64. */
 				uint64_t va_bits;
 				/** `TCR_EL1_T1SZ` on AArch64. */
@@ -221,6 +241,18 @@ struct drgn_program {
 				char *raw;
 				size_t raw_size;
 			} vmcoreinfo;
+			/**
+			 * Value of `THREAD_SIZE` in the kernel, or 0 if not
+			 * cached yet.
+			 */
+			uint64_t thread_size_cached;
+			/**
+			 * Value of `SECTIONS_PER_ROOT` in the kernel, or 0 if
+			 * not cached yet.
+			 */
+			uint64_t cached_sections_per_root;
+			/** Value of `ARCH_PFN_OFFSET` in the kernel. */
+			uint64_t arch_pfn_offset;
 			/*
 			 * Difference between a virtual address in the direct
 			 * mapping and the physical address it maps to.
@@ -228,6 +260,13 @@ struct drgn_program {
 			uint64_t direct_mapping_offset;
 			/** Cached value of `MOD_TEXT` in the kernel. */
 			uint64_t mod_text;
+			/** Cached array of per-cpu __irq_regs values */
+			uint64_t *irq_regs_cached;
+			/**
+			 * Whether @ref drgn_program::arch_pfn_offset has been
+			 * cached.
+			 */
+			bool arch_pfn_offset_cached;
 			/*
 			 * Whether @ref drgn_program::direct_mapping_offset has
 			 * been cached.
@@ -363,17 +402,6 @@ drgn_program_is_64_bit(struct drgn_program *prog, bool *ret)
 	return NULL;
 }
 
-static inline struct drgn_error *
-drgn_program_address_size(struct drgn_program *prog, uint8_t *ret)
-{
-	if (!prog->has_platform) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "program address size is not known");
-	}
-	*ret = drgn_platform_address_size(&prog->platform);
-	return NULL;
-}
-
 static inline struct drgn_error *
 drgn_program_address_mask(const struct drgn_program *prog, uint64_t *ret)
 {
