@@ -21,26 +21,6 @@
 #include "../symbol.h"
 #include "../vector.h"
 
-/* These were added in Python 3.7. */
-#ifndef Py_UNREACHABLE
-#define Py_UNREACHABLE() abort()
-#endif
-#ifndef Py_RETURN_RICHCOMPARE
-#define Py_RETURN_RICHCOMPARE(val1, val2, op)                               \
-    do {                                                                    \
-        switch (op) {                                                       \
-        case Py_EQ: if ((val1) == (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;  \
-        case Py_NE: if ((val1) != (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;  \
-        case Py_LT: if ((val1) < (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;   \
-        case Py_GT: if ((val1) > (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;   \
-        case Py_LE: if ((val1) <= (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;  \
-        case Py_GE: if ((val1) >= (val2)) Py_RETURN_TRUE; Py_RETURN_FALSE;  \
-        default:                                                            \
-            Py_UNREACHABLE();                                               \
-        }                                                                   \
-    } while (0)
-#endif
-
 #if PY_VERSION_HEX < 0x030900a1
 static inline PyObject *PyObject_CallNoArgs(PyObject *func)
 {
@@ -56,6 +36,10 @@ static inline PyObject *PyObject_CallOneArg(PyObject *callable, PyObject *arg)
 #define PyThreadState_GetUnchecked _PyThreadState_UncheckedGet
 #endif
 
+#if PY_VERSION_HEX < 0x030e00a5
+#define Py_HashPointer _Py_HashPointer
+#endif
+
 #define DRGNPY_PUBLIC __attribute__((__visibility__("default")))
 
 // PyLong_From* and PyLong_As* for stdint.h types. These use _Generic for
@@ -190,6 +174,7 @@ typedef struct {
 	PyObject_HEAD
 	struct drgn_program prog;
 	PyObject *cache;
+	PyObject *config;
 	/*
 	 * Set of objects that we need to hold a reference to during the
 	 * lifetime of the Program.
@@ -310,6 +295,7 @@ extern PyTypeObject ModuleIterator_type;
 extern PyTypeObject ModuleSectionAddressesIterator_type;
 extern PyTypeObject Module_type;
 extern PyTypeObject ObjectIterator_type;
+extern PyTypeObject ObjectNotFoundError_type;
 extern PyTypeObject Platform_type;
 extern PyTypeObject Program_type;
 extern PyTypeObject Register_type;
@@ -498,6 +484,8 @@ DrgnObject *drgnpy_linux_helper_task_thread_info(PyObject *self, PyObject *args,
 						 PyObject *kwds);
 PyObject *drgnpy_linux_helper_task_cpu(PyObject *self, PyObject *args,
 				       PyObject *kwds);
+PyObject *drgnpy_linux_helper_task_on_cpu(PyObject *self, PyObject *args,
+					  PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_xa_load(PyObject *self, PyObject *args,
 					PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_idr_find(PyObject *self, PyObject *args,
