@@ -197,6 +197,9 @@ static PyMethodDef drgn_methods[] = {
 	 METH_VARARGS | METH_KEYWORDS, drgn__linux_helper_task_thread_info_DOC},
 	{"_linux_helper_task_cpu", (PyCFunction)drgnpy_linux_helper_task_cpu,
 	 METH_VARARGS | METH_KEYWORDS, drgn__linux_helper_task_cpu_DOC},
+	{"_linux_helper_task_on_cpu",
+	 (PyCFunction)drgnpy_linux_helper_task_on_cpu,
+	 METH_VARARGS | METH_KEYWORDS, drgn__linux_helper_task_on_cpu_DOC},
 	{"_linux_helper_xa_load",
 	 (PyCFunction)drgnpy_linux_helper_xa_load,
 	 METH_VARARGS | METH_KEYWORDS},
@@ -251,17 +254,10 @@ static int add_type_aliases(PyObject *m)
 	if (!typing_Union)
 		return -1;
 
-	// This should be a subclass of typing.Protocol, but that is only
-	// available since Python 3.8.
-	PyObject *IntegerLike = PyType_FromSpec(&(PyType_Spec){
-		.name = "_drgn.IntegerLike",
-		.flags = Py_TPFLAGS_DEFAULT,
-		.slots = (PyType_Slot []){{0, NULL}},
-	});
-	if (!IntegerLike)
-		return -1;
-	if (PyModule_AddObject(m, "IntegerLike", IntegerLike) == -1) {
-		Py_DECREF(IntegerLike);
+	PyObject *typing_SupportsIndex =
+		PyObject_GetAttrString(typing_module, "SupportsIndex");
+	if (PyModule_AddObject(m, "IntegerLike", typing_SupportsIndex) == -1) {
+		Py_XDECREF(typing_SupportsIndex);
 		return -1;
 	}
 
@@ -342,6 +338,14 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 	if (add_type(m, &FaultError_type))
 		goto err;
 
+	ObjectNotFoundError_type.tp_base = (PyTypeObject *)PyExc_KeyError;
+	// KeyError.__str__() returns repr(args[0]). Use BaseException.__str__()
+	// instead, which returns str(args[0]).
+	ObjectNotFoundError_type.tp_str =
+		((PyTypeObject *)PyExc_BaseException)->tp_str;
+	if (add_type(m, &ObjectNotFoundError_type))
+		goto err;
+
 	PyObject *host_platform_obj = Platform_wrap(&drgn_host_platform);
 	if (!host_platform_obj)
 		goto err;
@@ -415,11 +419,6 @@ PyGILState_STATE drgn_initialize_python(bool *success_ret)
 		Py_InitializeEx(0);
 		// Note: we don't have a good place to call Py_Finalize(), so we
 		// don't call it.
-#if PY_VERSION_HEX < 0x03070000
-		// Py_Initialize() calls this for us since Python 3.7, and it
-		// was deprecated in Python 3.9.
-		PyEval_InitThreads();
-#endif
 		const char *env = getenv("PYTHONSAFEPATH");
 		if (!env || !env[0])
 			PyRun_SimpleString("import sys\nsys.path.insert(0, '')");
