@@ -783,26 +783,9 @@ drgn_get_initial_registers(struct drgn_program *prog, uint32_t tid,
 
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
 		bool on_cpu;
-		err = drgn_object_member_dereference(&tmp, &obj, "on_cpu");
-		if (!err) {
-			err = drgn_object_bool(&tmp, &on_cpu);
-			if (err)
-				return err;
-		} else if (err->code == DRGN_ERROR_LOOKUP) {
-			// The kernel must be !SMP. We have to check cpu_curr(0)
-			// instead.
-			drgn_error_destroy(err);
-			err = linux_helper_cpu_curr(&tmp, 0);
-			if (err)
-				return err;
-			int cmp;
-			err = drgn_object_cmp(&tmp, &obj, &cmp);
-			if (err)
-				return err;
-			on_cpu = cmp == 0;
-		} else {
+		err = linux_helper_task_on_cpu(&obj, &on_cpu);
+		if (err)
 			return err;
-		}
 		if (on_cpu) {
 			if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
 				return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
@@ -1184,6 +1167,15 @@ drgn_unwind_with_cfi(struct drgn_program *prog, struct drgn_cfi_row **row,
 							      ret_addr_regno,
 							      layout->offset,
 							      layout->size);
+	} else {
+		/* If we could not determine the return address despite having a
+		 * CFI rule for it, then we should try the fallback unwinder. */
+		struct drgn_cfi_rule rule;
+		drgn_cfi_row_get_register(*row, ret_addr_regno, &rule);
+		if (rule.kind != DRGN_CFI_RULE_UNDEFINED) {
+			drgn_register_state_destroy(unwound);
+			return &drgn_not_found;
+		}
 	}
 	*ret = unwound;
 	return NULL;
