@@ -34,26 +34,6 @@ LIBDRGN_PUBLIC void drgn_symbols_destroy(struct drgn_symbol **syms,
 	free(syms);
 }
 
-void drgn_symbol_from_elf(const char *name, uint64_t address,
-			  const GElf_Sym *elf_sym, struct drgn_symbol *ret)
-{
-	ret->name = name;
-	ret->name_lifetime = DRGN_LIFETIME_STATIC;
-	ret->lifetime = DRGN_LIFETIME_OWNED;
-	ret->address = address;
-	ret->size = elf_sym->st_size;
-	int binding = GELF_ST_BIND(elf_sym->st_info);
-	if (binding <= STB_WEAK || binding == STB_GNU_UNIQUE)
-		ret->binding = binding + 1;
-	else
-		ret->binding = DRGN_SYMBOL_BINDING_UNKNOWN;
-	int type = GELF_ST_TYPE(elf_sym->st_info);
-	if (type <= STT_TLS || type == STT_GNU_IFUNC)
-		ret->kind = type;
-	else
-		ret->kind = DRGN_SYMBOL_KIND_UNKNOWN;
-}
-
 struct drgn_error *
 drgn_symbol_copy(struct drgn_symbol *dst, struct drgn_symbol *src)
 {
@@ -142,6 +122,57 @@ drgn_symbol_result_builder_add(struct drgn_symbol_result_builder *builder,
 	return true;
 }
 
+static void drgn_symbol_from_elf(const char *name, uint64_t address,
+				 const GElf_Sym *elf_sym,
+				 struct drgn_symbol *ret)
+{
+	ret->name = name;
+	ret->name_lifetime = DRGN_LIFETIME_STATIC;
+	ret->lifetime = DRGN_LIFETIME_OWNED;
+	ret->address = address;
+	ret->size = elf_sym->st_size;
+	int binding = GELF_ST_BIND(elf_sym->st_info);
+	if (binding <= STB_WEAK || binding == STB_GNU_UNIQUE)
+		ret->binding = binding + 1;
+	else
+		ret->binding = DRGN_SYMBOL_BINDING_UNKNOWN;
+	int type = GELF_ST_TYPE(elf_sym->st_info);
+	if (type <= STT_TLS || type == STT_GNU_IFUNC)
+		ret->kind = type;
+	else
+		ret->kind = DRGN_SYMBOL_KIND_UNKNOWN;
+}
+
+bool
+drgn_symbol_result_builder_add_from_elf(struct drgn_symbol_result_builder *builder,
+					const char *name, uint64_t address,
+					const GElf_Sym *elf_sym)
+{
+	if (builder->one) {
+		// As an optimization, reuse the existing symbol allocation if
+		// we can.
+		if (!builder->single
+		    || builder->single->lifetime == DRGN_LIFETIME_STATIC) {
+			builder->single = malloc(sizeof(*builder->single));
+			if (!builder->single)
+				return false;
+		} else if (builder->single->name_lifetime == DRGN_LIFETIME_OWNED) {
+			free((char *)builder->single->name);
+		}
+		drgn_symbol_from_elf(name, address, elf_sym, builder->single);
+	} else {
+		struct drgn_symbol *sym = malloc(sizeof(*sym));
+		if (!sym)
+			return false;
+		drgn_symbol_from_elf(name, address, elf_sym, sym);
+		if (!symbolp_vector_append(&builder->vector, &sym)) {
+			free(sym);
+			return false;
+		}
+	}
+	return true;
+}
+
 LIBDRGN_PUBLIC size_t
 drgn_symbol_result_builder_count(const struct drgn_symbol_result_builder *builder)
 {
