@@ -130,44 +130,6 @@ def gen_python_mapping_file(mapping_path):
     os.rename(mapping_path + ".tmp", mapping_path)
 
 
-def iwyu_associated_header(path):
-    with open(path, "r") as f:
-        match = re.search(
-            r'^\s*#\s*include\s+"([^"]+)"\s+//\s+IWYU\s+pragma:\s+associated',
-            f.read(),
-            re.M,
-        )
-        if match:
-            return os.path.join(os.path.dirname(path), match.group(1))
-    if path.endswith(".c"):
-        return path[:-2] + ".h"
-    return None
-
-
-def ignore_line(path, state, line):
-    # include-what-you-use/include-what-you-use#969: iwyu recommends bogus
-    # forward declarations for the anonymous unions generated by
-    # BINARY_OP_SIGNED_2C.
-    if line.endswith("::;"):
-        return True
-
-    # include-what-you-use/include-what-you-use#971: drgn.h "exports" a forward
-    # declaration of several opaque types, but iwyu doesn't have such a notion.
-    if re.fullmatch(
-        r"struct drgn_(language|platform|program|register|stack_trace|symbol);", line
-    ):
-        paths = [path]
-        associated_header = iwyu_associated_header(path)
-        if associated_header is not None:
-            paths.append(associated_header)
-        for path in paths:
-            with open(path, "r") as f:
-                if re.search(r'^#include "(drgn.h|drgnpy.h)"', f.read(), re.M):
-                    return True
-
-    return False
-
-
 def main():
     parser = argparse.ArgumentParser(description="run include-what-you-use on drgn")
     parser.add_argument(
@@ -247,11 +209,7 @@ def main():
                     else:
                         header = None
                     lines.clear()
-                elif (
-                    line
-                    and state != "include_list"
-                    and not ignore_line(path, state, line)
-                ):
+                elif line and state != "include_list":
                     if header is not None:
                         print("\n" + header)
                         header = None
