@@ -3,18 +3,12 @@
 
 import gzip
 import re
-import unittest
 
-from _drgn_util.platform import NORMALIZED_MACHINE_NAME
 from drgn.helpers.linux.kconfig import get_kconfig
 from tests.linux_kernel import LinuxKernelTestCase
 
 
 class TestKconfig(LinuxKernelTestCase):
-    @unittest.skipIf(
-        NORMALIZED_MACHINE_NAME == "arm",
-        "get_kconfig() is broken on Arm due to elfutils bug",
-    )
     def test_get_kconfig(self):
         expected = {}
         try:
