@@ -5,14 +5,18 @@ import contextlib
 import ctypes
 import mmap
 import os
+from pathlib import Path
+import re
 import struct
 import sys
 import tempfile
 import unittest
 
 from _drgn_util.platform import NORMALIZED_MACHINE_NAME
-from drgn import NULL, FaultError
+from drgn import NULL, FaultError, ObjectNotFoundError
+from drgn.helpers.linux.device import dev_name
 from drgn.helpers.linux.mm import (
+    _MEMORY_BLOCK_STATE,
     PFN_PHYS,
     PHYS_PFN,
     PageCompound,
@@ -26,14 +30,19 @@ from drgn.helpers.linux.mm import (
     compound_head,
     compound_nr,
     compound_order,
+    decode_memory_block_state,
     decode_page_flags,
     environ,
     find_vmap_area,
     follow_page,
     follow_pfn,
     follow_phys,
+    for_each_memory_block,
+    for_each_valid_page_range,
     for_each_vma,
     for_each_vmap_area,
+    memory_block_size_bytes,
+    page_index,
     page_size,
     page_to_pfn,
     page_to_phys,
@@ -42,10 +51,13 @@ from drgn.helpers.linux.mm import (
     pfn_to_virt,
     phys_to_page,
     phys_to_virt,
+    task_rss,
     totalram_pages,
     virt_to_page,
     virt_to_pfn,
     virt_to_phys,
+    vm_commit_limit,
+    vm_memory_committed,
     vma_find,
     vmalloc_to_page,
     vmalloc_to_pfn,
@@ -55,11 +67,13 @@ from tests.linux_kernel import (
     LinuxKernelTestCase,
     fork_and_stop,
     iter_maps,
+    meminfo_field_in_pages,
     mlock,
     prng32,
     skip_if_highmem,
     skip_if_highpte,
     skip_unless_have_full_mm_support,
+    skip_unless_have_memory_hotplug,
     skip_unless_have_test_kmod,
 )
 
@@ -95,6 +109,10 @@ class TestMm(LinuxKernelTestCase):
                     ]
                 yield map, address, pfns
 
+    def test_page_index(self):
+        with self._pages() as (_, _, pfns):
+            self.assertEqual(page_index(pfn_to_page(self.prog, pfns[3])), 3)
+
     def test_page_flag_getters(self):
         with self._pages() as (map, _, pfns):
             page = pfn_to_page(self.prog, pfns[0])
@@ -158,6 +176,21 @@ class TestMm(LinuxKernelTestCase):
             page = pfn_to_page(self.prog, pfns[0])
             self.assertIn("PG_swapbacked", decode_page_flags(page))
 
+    @skip_unless_have_test_kmod
+    def test_for_each_valid_page_range(self):
+        expected_pfn = self.prog["drgn_test_pfn"].value_()
+        found_expected = False
+        for start_pfn, end_pfn, mem_map in for_each_valid_page_range(self.prog):
+            # We should be able to read all valid pages.
+            for page in mem_map[start_pfn:end_pfn]:
+                page._refcount.read_()
+            if start_pfn <= expected_pfn < end_pfn:
+                found_expected = True
+                self.assertEqual(
+                    mem_map + expected_pfn, self.prog["drgn_test_page"].read_()
+                )
+        self.assertTrue(found_expected)
+
     @skip_unless_have_test_kmod
     def test_PFN_PHYS(self):
         self.assertEqual(
@@ -443,12 +476,125 @@ class TestMm(LinuxKernelTestCase):
             )
 
     def test_totalram_pages(self):
-        with open("/proc/meminfo") as f:
-            lines = f.read().splitlines()
-            line = [line for line in lines if line.startswith("MemTotal:")][0]
-            parts = line.split()
-            self.assertEqual(parts[2], "kB")
+        self.assertEqual(totalram_pages(self.prog), meminfo_field_in_pages("MemTotal"))
+
+    def test_vm_commit_limit(self):
+        overcommit_kbytes_path = Path("/proc/sys/vm/overcommit_kbytes")
+        orig_overcommit_kbytes = int(overcommit_kbytes_path.read_text())
+        try:
+            for i in range(2):
+                if i == 1:
+                    if orig_overcommit_kbytes == 0:
+                        overcommit_kbytes = 1024 * 1024 * 1024
+                    else:
+                        overcommit_kbytes = 0
+                    overcommit_kbytes_path.write_text(str(overcommit_kbytes))
+                self.assertEqual(
+                    vm_commit_limit(self.prog), meminfo_field_in_pages("CommitLimit")
+                )
+        finally:
+            overcommit_kbytes_path.write_text(str(orig_overcommit_kbytes))
+
+    def test_vm_memory_committed(self):
+        self.assertAlmostEqual(
+            vm_memory_committed(self.prog),
+            meminfo_field_in_pages("Committed_AS"),
+            delta=1024 * 1024 * 1024,
+        )
+
+    def test_task_rss(self):
+        with fork_and_stop() as pid:
+            task = find_task(self.prog, pid)
+            rss_info = task_rss(self.prog, task)
 
-            proc_totalram = 1024 * int(parts[1])
             page_size = self.prog["PAGE_SIZE"].value_()
-            self.assertEqual(totalram_pages(self.prog) * page_size, proc_totalram)
+            # Get the relevant RSS counters, converting from kB to pages.
+            stats = {
+                key: int(value) * 1024 // page_size
+                for key, value in re.findall(
+                    r"^(VmRSS|RssAnon|RssFile|RssShmem|VmSwap):\s*([0-9]+)",
+                    Path(f"/proc/{pid}/status").read_text(),
+                    flags=re.MULTILINE,
+                )
+            }
+
+            # Before Linux kernel commit 82241a83cd15 ("mm: fix the inaccurate
+            # memory statistics issue for users") (in v6.16), the RSS counters
+            # in /proc/pid/meminfo are approximate due to batching, but the
+            # helpers are exact.
+            if hasattr(task, "rss_stat"):
+                # Before Linux kernel commit f1a7941243c10 ("mm: convert mm's
+                # rss stats into percpu_counter") (in v6.2), there is a
+                # per-thread counter that only gets synced to the main counter
+                # every TASK_RSS_EVENTS_THRESH (64) page faults. Each fault can
+                # map in multiple pages based on fault_around_bytes. So, the
+                # maximum error is nr_threads * 64 * (fault_around_bytes / PAGE_SIZE).
+                delta = (
+                    len(os.listdir(f"/proc/{pid}/task"))
+                    * 64
+                    * (self.prog["fault_around_bytes"].value_() // page_size)
+                )
+            else:
+                # Between that and Linux kernel commit 82241a83cd15 ("mm: fix
+                # the inaccurate memory statistics issue for users") (in
+                # v6.16), the kernel code uses percpu_counter_read_positive(),
+                # so the maximum error is nr_cpus * percpu_counter_batch.
+                try:
+                    percpu_counter_batch = self.prog["percpu_counter_batch"].value_()
+                except ObjectNotFoundError:
+                    percpu_counter_batch = 32
+                delta = percpu_counter_batch * os.cpu_count()
+
+            self.assertAlmostEqual(rss_info.file, stats["RssFile"], delta=delta)
+            self.assertAlmostEqual(rss_info.anon, stats["RssAnon"], delta=delta)
+            self.assertAlmostEqual(
+                rss_info.shmem, stats.get("RssShmem", 0), delta=delta
+            )
+            self.assertAlmostEqual(rss_info.swap, stats["VmSwap"], delta=delta)
+            # VmRSS is the sum of three counters, so it has triple the error
+            # margin.
+            self.assertAlmostEqual(rss_info.total, stats["VmRSS"], delta=delta * 3)
+
+    @skip_unless_have_memory_hotplug
+    def test_for_each_memory_block(self):
+        self.assertCountEqual(
+            [
+                dev_name(mem.dev.address_of_())
+                for mem in for_each_memory_block(self.prog)
+            ],
+            os.listdir(b"/sys/bus/memory/devices"),
+        )
+
+    @skip_unless_have_test_kmod
+    def test_memory_block_states(self):
+        for value, name in _MEMORY_BLOCK_STATE.items():
+            with self.subTest(state=name):
+                try:
+                    expected = self.prog["drgn_test_" + name].value_()
+                except ObjectNotFoundError:
+                    self.skipTest(f"{name} is not defined")
+                self.assertEqual(value, expected)
+
+    @skip_unless_have_memory_hotplug
+    def test_decode_memory_block_state(self):
+        mem = next(iter(for_each_memory_block(self.prog)))
+        self.assertEqual(
+            decode_memory_block_state(mem)
+            .replace("MEM_", "")
+            .lower()
+            .replace("_", "-"),
+            (
+                Path("/sys/bus/memory/devices")
+                / os.fsdecode(dev_name(mem.dev.address_of_()))
+                / "state"
+            )
+            .read_text()
+            .strip(),
+        )
+
+    @skip_unless_have_memory_hotplug
+    def test_memory_block_size_bytes(self):
+        self.assertEqual(
+            memory_block_size_bytes(self.prog),
+            int(Path("/sys/devices/system/memory/block_size_bytes").read_text(), 16),
+        )
