@@ -24,9 +24,12 @@
 #else
 #define HAVE_MAPLE_TREE 0
 #endif
+#include <linux/memory.h>
 #include <linux/mm.h>
+#include <linux/mmzone.h>
 #include <linux/module.h>
 #include <linux/netdevice.h>
+#include <linux/nodemask.h>
 #include <linux/plist.h>
 #include <linux/radix-tree.h>
 #include <linux/rbtree.h>
@@ -63,6 +66,12 @@
 #define HAVE_PAGE_POOL 0
 #endif
 
+// Architecture-specific includes for triggering NMIs for stack traces
+#ifdef __x86_64__
+#include <asm/apic.h>
+#include <asm/nmi.h>
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 18, 0)
 // These were added in b9ff604cff11 ("timekeeping: Add
 // ktime_get_coarse_with_offset") (in v4.18-rc1).
@@ -125,6 +134,60 @@ static u32 drgn_test_prng32(u32 x)
 	return x;
 }
 
+// constants
+
+unsigned long drgn_test_THREAD_SIZE;
+#ifdef NR_SECTION_ROOTS
+unsigned long drgn_test_NR_SECTION_ROOTS;
+#endif
+#ifdef SECTIONS_PER_ROOT
+unsigned long drgn_test_SECTIONS_PER_ROOT;
+#endif
+#ifdef SECTION_SIZE_BITS
+unsigned long drgn_test_SECTION_SIZE_BITS;
+#endif
+#ifdef MAX_PHYSMEM_BITS
+unsigned long drgn_test_MAX_PHYSMEM_BITS;
+#endif
+#ifdef CONFIG_FLATMEM
+unsigned long drgn_test_ARCH_PFN_OFFSET;
+#endif
+
+int drgn_test_MEM_ONLINE = MEM_ONLINE;
+int drgn_test_MEM_GOING_OFFLINE = MEM_GOING_OFFLINE;
+int drgn_test_MEM_OFFLINE = MEM_OFFLINE;
+int drgn_test_MEM_GOING_ONLINE = MEM_GOING_ONLINE;
+int drgn_test_MEM_CANCEL_ONLINE = MEM_CANCEL_ONLINE;
+int drgn_test_MEM_CANCEL_OFFLINE = MEM_CANCEL_OFFLINE;
+#ifdef MEM_PREPARE_ONLINE
+int drgn_test_MEM_PREPARE_ONLINE = MEM_PREPARE_ONLINE;
+#endif
+#ifdef MEM_FINISH_OFFLINE
+int drgn_test_MEM_FINISH_OFFLINE = MEM_FINISH_OFFLINE;
+#endif
+
+static void drgn_test_constants_init(void)
+{
+	// Some of these aren't actually compile-time constants, so we
+	// initialize all of them at runtime.
+	drgn_test_THREAD_SIZE = THREAD_SIZE;
+#ifdef NR_SECTION_ROOTS
+	drgn_test_NR_SECTION_ROOTS = NR_SECTION_ROOTS;
+#endif
+#ifdef SECTIONS_PER_ROOT
+	drgn_test_SECTIONS_PER_ROOT = SECTIONS_PER_ROOT;
+#endif
+#ifdef SECTION_SIZE_BITS
+	drgn_test_SECTION_SIZE_BITS = SECTION_SIZE_BITS;
+#endif
+#ifdef MAX_PHYSMEM_BITS
+	drgn_test_MAX_PHYSMEM_BITS = MAX_PHYSMEM_BITS;
+#endif
+#ifdef CONFIG_FLATMEM
+	drgn_test_ARCH_PFN_OFFSET = ARCH_PFN_OFFSET;
+#endif
+}
+
 // list
 
 LIST_HEAD(drgn_test_empty_list);
@@ -462,6 +525,8 @@ static void drgn_test_maple_tree_exit(void) {}
 // mm
 
 const int drgn_test_vmap_stack_enabled = IS_ENABLED(CONFIG_VMAP_STACK);
+const int drgn_test_slab_stack_enabled =
+	!IS_ENABLED(CONFIG_VMAP_STACK) && THREAD_SIZE < PAGE_SIZE;
 void *drgn_test_va;
 phys_addr_t drgn_test_pa;
 unsigned long drgn_test_pfn;
@@ -471,10 +536,55 @@ void *drgn_test_vmalloc_va;
 unsigned long drgn_test_vmalloc_pfn;
 struct page *drgn_test_vmalloc_page;
 
+#ifdef CONFIG_SPARSEMEM
+unsigned long drgn_test_section_nr;
+unsigned long drgn_test_section_pfn;
+struct mem_section *drgn_test_mem_section;
+struct page *drgn_test_section_mem_map;
+struct page *drgn_test_section_decoded_mem_map;
+
+unsigned long drgn_test_SECTION_MARKED_PRESENT = SECTION_MARKED_PRESENT;
+struct mem_section *drgn_test_present_section = &(struct mem_section){
+	.section_mem_map = SECTION_MARKED_PRESENT,
+};
+
+unsigned long drgn_test_SECTION_HAS_MEM_MAP = SECTION_HAS_MEM_MAP;
+struct mem_section *drgn_test_valid_section = &(struct mem_section){
+	.section_mem_map = SECTION_HAS_MEM_MAP,
+};
+
+#ifdef SECTION_IS_ONLINE
+unsigned long drgn_test_SECTION_IS_ONLINE = SECTION_IS_ONLINE;
+struct mem_section *drgn_test_online_section = &(struct mem_section){
+	.section_mem_map = SECTION_IS_ONLINE,
+};
+#endif
+
+#ifdef SECTION_IS_EARLY
+unsigned long drgn_test_SECTION_IS_EARLY = SECTION_IS_EARLY;
+struct mem_section *drgn_test_early_section = &(struct mem_section){
+	.section_mem_map = SECTION_IS_EARLY,
+};
+#endif
+
+#ifdef SECTION_TAINT_ZONE_DEVICE
+unsigned long drgn_test_SECTION_TAINT_ZONE_DEVICE = SECTION_TAINT_ZONE_DEVICE;
+#endif
+
+struct {
+	unsigned long nr;
+	struct mem_section *section;
+} *drgn_test_present_sections;
+size_t drgn_test_num_present_sections;
+#endif
+
 static int drgn_test_mm_init(void)
 {
 	u32 fill;
 	size_t i;
+#ifdef CONFIG_SPARSEMEM
+	size_t n;
+#endif
 
 	drgn_test_page = alloc_page(GFP_KERNEL);
 	if (!drgn_test_page)
@@ -496,6 +606,42 @@ static int drgn_test_mm_init(void)
 		return -ENOMEM;
 	drgn_test_vmalloc_pfn = vmalloc_to_pfn(drgn_test_vmalloc_va);
 	drgn_test_vmalloc_page = vmalloc_to_page(drgn_test_vmalloc_va);
+
+#ifdef CONFIG_SPARSEMEM
+	drgn_test_section_nr = pfn_to_section_nr(drgn_test_pfn);
+	drgn_test_section_pfn = section_nr_to_pfn(drgn_test_section_nr);
+	drgn_test_mem_section = __nr_to_section(drgn_test_section_nr);
+	drgn_test_section_mem_map = __section_mem_map_addr(drgn_test_mem_section);
+	// sparse_decode_mem_map() isn't exported, so we do the equivalent
+	// ourselves.
+	drgn_test_section_decoded_mem_map =
+		drgn_test_section_mem_map + drgn_test_section_pfn;
+
+	// __highest_present_section_nr is not exported, so we can't use
+	// for_each_present_section_nr().
+	for (i = 0, n = 0; i < NR_MEM_SECTIONS; i++) {
+		if (present_section_nr(i))
+			n++;
+	}
+
+	drgn_test_present_sections =
+		kmalloc_array(n, sizeof(drgn_test_present_sections[0]),
+			      GFP_KERNEL);
+	if (!drgn_test_present_sections)
+		return -ENOMEM;
+
+	for (i = 0; i < NR_MEM_SECTIONS; i++) {
+		if (present_section_nr(i)) {
+			drgn_test_present_sections[drgn_test_num_present_sections].nr = i;
+			drgn_test_present_sections[drgn_test_num_present_sections].section =
+				__nr_to_section(i);
+			drgn_test_num_present_sections++;
+			if (drgn_test_num_present_sections >= n)
+				break;
+		}
+	}
+#endif
+
 	return 0;
 }
 
@@ -508,6 +654,17 @@ static void drgn_test_mm_exit(void)
 		__free_pages(drgn_test_page, 0);
 }
 
+// mmzone
+
+int drgn_test_nid;
+struct pglist_data *drgn_test_pgdat;
+
+static void drgn_test_mmzone_init(void)
+{
+	drgn_test_nid = first_online_node;
+	drgn_test_pgdat = NODE_DATA(drgn_test_nid);
+}
+
 // net
 
 struct net_device *drgn_test_netdev;
@@ -590,9 +747,23 @@ static void drgn_test_page_pool_exit(void)
 
 DEFINE_PER_CPU(u32, drgn_test_percpu_static);
 u32 __percpu *drgn_test_percpu_dynamic;
+struct percpu_counter drgn_test_percpu_counter;
+struct percpu_counter drgn_test_percpu_counter_negative;
+
+struct drgn_test_percpu_struct {
+	int cpu;
+	int i;
+};
+
+DEFINE_PER_CPU(struct drgn_test_percpu_struct, drgn_test_percpu_structs);
+
+typedef struct drgn_test_percpu_struct drgn_test_percpu_array[3];
+
+DEFINE_PER_CPU(drgn_test_percpu_array, drgn_test_percpu_arrays);
 
 static int drgn_test_percpu_init(void)
 {
+	int ret;
 	int cpu;
 	u32 static_seed = drgn_test_prng32_seed("PCPU");
 	u32 dynamic_seed = drgn_test_prng32_seed("pcpu");
@@ -602,16 +773,46 @@ static int drgn_test_percpu_init(void)
 		return -ENOMEM;
 	// Initialize the per-cpu variables with a PRNG sequence.
 	for_each_possible_cpu(cpu) {
+		int i;
+
 		static_seed = drgn_test_prng32(static_seed);
 		per_cpu(drgn_test_percpu_static, cpu) = static_seed;
 		dynamic_seed = drgn_test_prng32(dynamic_seed);
 		*per_cpu_ptr(drgn_test_percpu_dynamic, cpu) = dynamic_seed;
+
+		per_cpu(drgn_test_percpu_structs, cpu) =
+			(struct drgn_test_percpu_struct){
+				.cpu = cpu,
+			};
+
+		for (i = 0; i < 3; i++) {
+			per_cpu(drgn_test_percpu_arrays, cpu)[i] =
+				(struct drgn_test_percpu_struct){
+					.cpu = cpu,
+					.i = i,
+				};
+		}
 	}
+
+	ret = percpu_counter_init(&drgn_test_percpu_counter,
+				  10, GFP_KERNEL);
+	if (ret)
+		return ret;
+	percpu_counter_add(&drgn_test_percpu_counter, 3);
+
+	ret = percpu_counter_init(&drgn_test_percpu_counter_negative,
+				  33, GFP_KERNEL);
+	if (ret)
+		return ret;
+	percpu_counter_sub(&drgn_test_percpu_counter_negative, 99);
+
 	return 0;
 }
 
 static void drgn_test_percpu_exit(void)
 {
+	percpu_counter_destroy(&drgn_test_percpu_counter_negative);
+	percpu_counter_destroy(&drgn_test_percpu_counter);
 	free_percpu(drgn_test_percpu_dynamic);
 }
 
@@ -629,6 +830,11 @@ struct drgn_test_rb_entry drgn_test_rb_entries[4];
 
 struct rb_node drgn_test_empty_rb_node;
 
+struct drgn_test_rbtree_container_struct {
+	struct drgn_test_rb_entry entries[2];
+	struct rb_root root;
+} drgn_test_rbtree_container;
+
 struct rb_root drgn_test_rbtree_with_equal = RB_ROOT;
 struct drgn_test_rb_entry drgn_test_rb_entries_with_equal[4];
 
@@ -682,6 +888,12 @@ static void drgn_test_rbtree_init(void)
 	}
 	RB_CLEAR_NODE(&drgn_test_empty_rb_node);
 
+	for (i = 0; i < ARRAY_SIZE(drgn_test_rbtree_container.entries); i++) {
+		drgn_test_rbtree_container.entries[i].value = i;
+		drgn_test_rbtree_insert(&drgn_test_rbtree_container.root,
+					&drgn_test_rbtree_container.entries[i]);
+	}
+
 	// Red-black tree with entries that compare equal to each other.
 	for (i = 0; i < ARRAY_SIZE(drgn_test_rb_entries_with_equal); i++) {
 		drgn_test_rb_entries_with_equal[i].value = i / 2;
@@ -791,7 +1003,14 @@ static void drgn_test_slab_ctor(void *arg)
 
 static int drgn_test_slab_init(void)
 {
-	size_t i;
+	size_t num_tmp_objs;
+	size_t i, j;
+
+	// We want objects in the drgn_test_small cache to be spread out over
+	// multiple slabs. To accomplish that, we allocate a bunch of temporary
+	// objects in the middle of the allocations we intend to keep, then free
+	// the temporary objects.
+	num_tmp_objs = PAGE_SIZE / sizeof(struct drgn_test_small_slab_object);
 
 	drgn_test_small_kmem_cache =
 		kmem_cache_create("drgn_test_small",
@@ -801,11 +1020,42 @@ static int drgn_test_slab_init(void)
 	if (!drgn_test_small_kmem_cache)
 		return -ENOMEM;
 	for (i = 0; i < ARRAY_SIZE(drgn_test_small_slab_objects); i++) {
+		const bool alloc_tmp_objs =
+			i == ARRAY_SIZE(drgn_test_small_slab_objects) / 2;
+		void **tmp_objs;
+		int error = 0;
+
+		if (alloc_tmp_objs) {
+			tmp_objs = kmalloc_array(num_tmp_objs,
+						 sizeof(*tmp_objs), GFP_KERNEL);
+			if (!tmp_objs)
+				return -ENOMEM;
+			for (j = 0; j < num_tmp_objs; j++) {
+				tmp_objs[j] = kmem_cache_alloc(drgn_test_small_kmem_cache,
+							       GFP_KERNEL);
+				// We check for allocation failures below to
+				// avoid duplicating cleanup code.
+			}
+		}
+
 		drgn_test_small_slab_objects[i] =
 			kmem_cache_alloc(drgn_test_small_kmem_cache,
 					 GFP_KERNEL);
 		if (!drgn_test_small_slab_objects[i])
+			error = -ENOMEM;
+
+		if (alloc_tmp_objs) {
+			for (j = 0; j < num_tmp_objs; j++) {
+				if (!tmp_objs[j])
+					error = -ENOMEM;
+				kmem_cache_free(drgn_test_small_kmem_cache,
+						tmp_objs[j]);
+			}
+			kfree(tmp_objs);
+		}
+		if (error)
 			return -ENOMEM;
+
 		drgn_test_small_slab_objects[i]->value = i;
 	}
 	drgn_test_big_kmem_cache =
@@ -1008,11 +1258,11 @@ static inline void drgn_test_get_pt_regs(struct pt_regs *regs)
 #endif
 }
 
- __attribute__((__optimize__("O0")))
+__attribute__((__noipa__))
 static void drgn_test_kthread_fn3(void)
 {
 	// Create some local variables for the test cases to use. Use volatile
-	// to make doubly sure that they aren't optimized out.
+	// to prevent them from being optimized out.
 	volatile int a, b, c;
 	volatile struct drgn_test_small_slab_object *slab_object;
 
@@ -1058,14 +1308,15 @@ static void drgn_test_kthread_fn3(void)
 	__asm__ __volatile__ ("" : : "r" (&slab_object) : "memory");
 }
 
- __attribute__((__optimize__("O0")))
+__attribute__((__noipa__))
 static void drgn_test_kthread_fn2(void)
 {
 	drgn_test_kthread_fn3();
+	barrier(); // Prevent tail call.
 }
 
- __attribute__((__optimize__("O0")))
-static int drgn_test_kthread_fn(void *arg)
+__attribute__((__noipa__))
+static noinline int drgn_test_kthread_fn(void *arg)
 {
 	drgn_test_kthread_fn2();
 	return 0;
@@ -1171,8 +1422,15 @@ DEFINE_XARRAY(drgn_test_xarray_multi_index);
 DEFINE_XARRAY(drgn_test_xarray_zero_entry);
 DEFINE_XARRAY(drgn_test_xarray_zero_entry_at_zero);
 DEFINE_XARRAY(drgn_test_xarray_value);
+DEFINE_XARRAY(drgn_test_xarray_pointers);
 void *drgn_test_xa_zero_entry;
 
+struct drgn_test_xarray_entry {
+	int value;
+};
+
+struct drgn_test_xarray_entry drgn_test_xarray_entries[4];
+
 static int drgn_test_xa_store_order(struct xarray *xa, unsigned long index,
 				    unsigned order, void *entry, gfp_t gfp)
 {
@@ -1192,6 +1450,7 @@ static int drgn_test_xarray_init(void)
 #if HAVE_XARRAY
 	void *entry;
 	int ret;
+	size_t i;
 
 	drgn_test_xa_zero_entry = XA_ZERO_ENTRY;
 
@@ -1237,6 +1496,14 @@ static int drgn_test_xarray_init(void)
 	if (xa_is_err(entry))
 		return xa_err(entry);
 
+	for (i = 0; i < ARRAY_SIZE(drgn_test_xarray_entries); i++) {
+		drgn_test_xarray_entries[i].value = i;
+		entry = xa_store(&drgn_test_xarray_pointers, i,
+				 &drgn_test_xarray_entries[i], GFP_KERNEL);
+		if (xa_is_err(entry))
+			return xa_err(entry);
+	}
+
 #endif
 
 	return 0;
@@ -1306,6 +1573,11 @@ static struct task_struct *drgn_test_waitq_kthread;
 static wait_queue_head_t drgn_test_waitq;
 static wait_queue_head_t drgn_test_empty_waitq;
 
+struct drgn_test_waitq_container_struct {
+	int x;
+	wait_queue_head_t waitq;
+} drgn_test_waitq_container;
+
 static int drgn_test_waitq_kthread_fn(void *arg)
 {
 	wait_event_interruptible(drgn_test_waitq, kthread_should_stop());
@@ -1316,6 +1588,7 @@ static int drgn_test_waitq_init(void)
 {
 	init_waitqueue_head(&drgn_test_waitq);
 	init_waitqueue_head(&drgn_test_empty_waitq);
+	init_waitqueue_head(&drgn_test_waitq_container.waitq);
 
 	drgn_test_waitq_kthread = kthread_create(drgn_test_waitq_kthread_fn,
 						 NULL,
@@ -1366,6 +1639,24 @@ char drgn_kmodify_test_memory[16];
 int drgn_kmodify_test_int;
 int *drgn_kmodify_test_int_ptr;
 
+unsigned long drgn_kmodify_test_bitmap[2];
+module_param_array_named(kmodify_bitmap, drgn_kmodify_test_bitmap, ulong, NULL,
+			 0600);
+
+struct {
+	unsigned int expect0_1 : 8;
+	unsigned int byte_aligned : 8;
+	unsigned int expect1_1 : 1;
+	unsigned int bit : 1;
+	unsigned int expect1_2 : 1;
+	unsigned int expect0_2 : 1;
+	unsigned int two_bits : 2;
+	unsigned int unaligned : 8;
+} drgn_kmodify_test_bit_field = {
+	.expect1_1 = 1,
+	.expect1_2 = 1,
+};
+
 #define DEFINE_KMODIFY_TEST_RETURN(name, return_type, return_value)	\
 int drgn_kmodify_test_##name##_called = 0;				\
 return_type drgn_kmodify_test_##name(void);				\
@@ -1504,19 +1795,53 @@ DEFINE_KMODIFY_TEST_ARGS(
 
 #ifdef CONFIG_SYSFS
 
-// Crash from an IRQ handler on architectures where drgn supports unwinding
-// through IRQ handlers.
+// Crash from an NMI + IRQ handler on architectures where drgn supports
+// unwinding through them.
 #ifdef __x86_64__
+#define DRGN_TEST_NMI_CRASH
+#elif defined(__aarch64__)
 #define DRGN_TEST_IRQ_CRASH
 #endif
 
-static __noreturn noinline_for_stack void drgn_test_crash_func(struct irq_work *work)
+
+// NMI test depends on IRQ
+#ifdef DRGN_TEST_NMI_CRASH
+#define DRGN_TEST_IRQ_CRASH
+#endif
+
+static __noreturn noinline_for_stack void drgn_test_crash_func(void)
 {
 	panic("drgn_test\n");
 }
 
+
+#ifdef DRGN_TEST_NMI_CRASH
+static bool drgn_panic = false;
+
+static noinline_for_stack int drgn_test_nmi_handler(unsigned int cmd, struct pt_regs *regs)
+{
+	if (drgn_panic) {
+		drgn_test_crash_func();
+		return NMI_HANDLED; /* lol */
+	}
+
+	return NMI_DONE;
+}
+
+static noinline_for_stack void drgn_test_crash_irq_work_fn(struct irq_work *work)
+{
+	drgn_panic = true;
+	apic->send_IPI_mask(cpumask_of(smp_processor_id()), NMI_VECTOR);
+}
+#elif defined(DRGN_TEST_IRQ_CRASH)
+static noinline_for_stack void drgn_test_crash_irq_work_fn(struct irq_work *work)
+{
+	drgn_test_crash_func();
+}
+#endif
+
 #ifdef DRGN_TEST_IRQ_CRASH
-static DEFINE_IRQ_WORK(drgn_test_crash_irq_work, drgn_test_crash_func);
+static DEFINE_IRQ_WORK(drgn_test_crash_irq_work, drgn_test_crash_irq_work_fn);
 #endif
 
 static ssize_t drgn_test_crash_store(struct kobject *kobj,
@@ -1537,7 +1862,17 @@ static ssize_t drgn_test_crash_store(struct kobject *kobj,
 	// Spin until we get interrupted and crash.
 	while (1);
 #else
-	drgn_test_crash_func(NULL);
+	drgn_test_crash_func();
+#endif
+}
+
+static int drgn_test_crash_init(void)
+{
+#ifdef DRGN_TEST_NMI_CRASH
+	return register_nmi_handler(NMI_LOCAL, drgn_test_nmi_handler,
+	                            0, "drgn_panic");
+#else
+	return 0;
 #endif
 }
 
@@ -1617,6 +1952,20 @@ static inline int drgn_test_sysfs_init(void) { return 0; }
 static inline void drgn_test_sysfs_exit(void) {}
 #endif
 
+// types
+
+union drgn_test_union {
+	u32 u;
+	s32 s;
+};
+union drgn_test_union drgn_test_union_var;
+
+typedef union {
+	u64 u;
+	s64 s;
+} drgn_test_anonymous_union;
+drgn_test_anonymous_union drgn_test_anonymous_union_var;
+
 static void drgn_test_exit(void)
 {
 	drgn_test_sysfs_exit();
@@ -1637,6 +1986,7 @@ static int __init drgn_test_init(void)
 {
 	int ret;
 
+	drgn_test_constants_init();
 	drgn_test_list_init();
 	drgn_test_llist_init();
 	drgn_test_plist_init();
@@ -1646,6 +1996,7 @@ static int __init drgn_test_init(void)
 	ret = drgn_test_mm_init();
 	if (ret)
 		goto out;
+	drgn_test_mmzone_init();
 	ret = drgn_test_net_init();
 	if (ret)
 		goto out;
@@ -1676,6 +2027,9 @@ static int __init drgn_test_init(void)
 	if (ret)
 		goto out;
 	ret = drgn_test_sysfs_init();
+	if (ret)
+		goto out;
+	ret = drgn_test_crash_init();
 out:
 	if (ret)
 		drgn_test_exit();
@@ -1686,3 +2040,4 @@ module_init(drgn_test_init);
 module_exit(drgn_test_exit);
 
 MODULE_LICENSE("GPL");
+MODULE_DESCRIPTION("Module for testing drgn");
