@@ -26,6 +26,7 @@ from tests.linux_kernel import (
     MS_NOEXEC,
     MS_NOSUID,
     LinuxKernelTestCase,
+    fallocate,
     fork_and_stop,
     iter_mounts,
     losetup,
@@ -212,7 +213,7 @@ class TestFsRefs(LinuxKernelTestCase):
         disk = os.environ["DRGN_TEST_DISK"]
         for fstype, mkfs in (
             ("ext2", ("mke2fs", "-qF")),
-            ("btrfs", ("mkfs.btrfs", "-qf")),
+            ("btrfs", ("mkfs.btrfs", "-qf", "-s", str(mmap.PAGESIZE))),
         ):
             with self.subTest(fstype=fstype):
                 subprocess.check_call([*mkfs, disk])
@@ -236,13 +237,13 @@ class TestFsRefs(LinuxKernelTestCase):
     def test_btrfs_subvolume(self):
         disk = os.environ["DRGN_TEST_DISK"]
         with contextlib.ExitStack() as exit_stack:
-            subprocess.check_call(["mkfs.btrfs", "-qf", disk])
+            subprocess.check_call(["mkfs.btrfs", "-qf", "-s", str(mmap.PAGESIZE), disk])
 
             mount(disk, self._tmp, "btrfs")
             exit_stack.callback(umount, self._tmp)
 
             subvol = self._tmp / "subvol"
-            subprocess.check_call(["btrfs", "subvolume", "create", subvol])
+            subprocess.check_call(["btrfs", "-q", "subvolume", "create", subvol])
 
             top_file = self._tmp / "file"
             top_fd = os.open(top_file, os.O_CREAT | os.O_WRONLY, 0o600)
@@ -375,7 +376,8 @@ class TestFsRefs(LinuxKernelTestCase):
             exit_stack.callback(umount, self._tmp)
 
             path = self._tmp / "swap_file"
-            mkswap(path, 1024 * 1024)
+            fallocate(path, 0, 1024 * 1024)
+            mkswap(path)
             swapon(path)
             exit_stack.callback(swapoff, path)
 
