@@ -15,6 +15,7 @@ from drgn import (
     Language,
     NoDefaultProgramError,
     Object,
+    ObjectNotFoundError,
     Platform,
     PlatformFlags,
     Program,
@@ -70,6 +71,7 @@ class TestProgram(TestCase):
         self.assertIsNone(prog.platform)
         self.assertFalse(prog.flags & ProgramFlags.IS_LIVE)
         prog.set_pid(os.getpid())
+        self.assertIsNone(prog.core_dump_path)
         self.assertEqual(prog.platform, host_platform)
         self.assertTrue(prog.flags & ProgramFlags.IS_LIVE)
         self.assertRaisesRegex(
@@ -104,46 +106,65 @@ class TestProgram(TestCase):
 
         self.assertEqual(prog.read(ctypes.addressof(buf), len(data)), data)
 
-    def test_lookup_error(self):
+    def test_object_not_found_error(self):
         prog = mock_program()
-        self.assertRaisesRegex(
-            LookupError, "^could not find constant 'foo'$", prog.constant, "foo"
-        )
-        self.assertRaisesRegex(
-            LookupError,
-            "^could not find constant 'foo' in 'foo.c'$",
-            prog.constant,
-            "foo",
-            "foo.c",
-        )
-        self.assertRaisesRegex(
-            LookupError, "^could not find function 'foo'$", prog.function, "foo"
-        )
-        self.assertRaisesRegex(
-            LookupError,
-            "^could not find function 'foo' in 'foo.c'$",
-            prog.function,
-            "foo",
-            "foo.c",
-        )
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find constant 'foo'$"
+        ) as cm:
+            prog.constant("foo")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find constant 'foo' in 'foo.c'$"
+        ) as cm:
+            prog.constant("foo", "foo.c")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find function 'foo'$"
+        ) as cm:
+            prog.function("foo")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find function 'foo' in 'foo.c'$"
+        ) as cm:
+            prog.function("foo", "foo.c")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find variable 'foo'$"
+        ) as cm:
+            prog.variable("foo")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find variable 'foo' in 'foo.c'$"
+        ) as cm:
+            prog.variable("foo", "foo.c")
+        self.assertEqual(cm.exception.name, "foo")
+
+        with self.assertRaisesRegex(
+            ObjectNotFoundError, "^could not find 'foo'$"
+        ) as cm:
+            prog["foo"]
+        self.assertEqual(cm.exception.name, "foo")
+
+        # If name isn't a string, prog.object(name) should raise TypeError, and
+        # prog[name] should raise KeyError (not ObjectNotFoundError).
+        self.assertRaises(TypeError, prog.object, 9)
+        with self.assertRaises(KeyError) as cm:
+            prog[9]
+        self.assertIs(type(cm.exception), KeyError)
+
+    def test_type_lookup_error(self):
+        prog = mock_program()
+
         self.assertRaisesRegex(LookupError, "^could not find 'foo'$", prog.type, "foo")
         self.assertRaisesRegex(
             LookupError, "^could not find 'foo' in 'foo.c'$", prog.type, "foo", "foo.c"
         )
-        self.assertRaisesRegex(
-            LookupError, "^could not find variable 'foo'$", prog.variable, "foo"
-        )
-        self.assertRaisesRegex(
-            LookupError,
-            "^could not find variable 'foo' in 'foo.c'$",
-            prog.variable,
-            "foo",
-            "foo.c",
-        )
-        # prog[key] should raise KeyError instead of LookupError.
-        self.assertRaises(KeyError, prog.__getitem__, "foo")
-        # Even for non-strings.
-        self.assertRaises(KeyError, prog.__getitem__, 9)
 
     def test_flags(self):
         self.assertIsInstance(mock_program().flags, ProgramFlags)
@@ -166,6 +187,13 @@ class TestProgram(TestCase):
         with self.assertRaises(AttributeError):
             del Program().language
 
+    def test_address_size(self):
+        self.assertEqual(mock_program().address_size(), 8)
+        self.assertEqual(mock_program(MOCK_32BIT_PLATFORM).address_size(), 4)
+        self.assertRaisesRegex(
+            ValueError, "address size is not known", Program().address_size
+        )
+
 
 class TestMemory(TestCase):
     def test_simple_read(self):
@@ -174,6 +202,27 @@ class TestMemory(TestCase):
         self.assertEqual(prog.read(0xFFFF0000, len(data)), data)
         self.assertEqual(prog.read(0xA0, len(data), True), data)
 
+    def test_read_c_string(self):
+        data = b"hello, world"
+        prog = mock_program(
+            segments=[MockMemorySegment(data + b"\0", 0xFFFF0000, 0xA0)]
+        )
+        self.assertEqual(prog.read_c_string(0xFFFF0000), data)
+        self.assertEqual(prog.read_c_string(0xA0, True), data)
+
+    def test_read_c_string_max_size(self):
+        for data in (b"hello, world", b"hell", b"he"):
+            with self.subTest(data=data):
+                prog = mock_program(
+                    segments=[
+                        MockMemorySegment(
+                            data + (b"\0" if len(data) < 4 else b""), 0xFFFF0000, 0xA0
+                        )
+                    ]
+                )
+                self.assertEqual(prog.read_c_string(0xFFFF0000, max_size=4), data[:4])
+                self.assertEqual(prog.read_c_string(0xA0, True, max_size=4), data[:4])
+
     def test_read_unsigned(self):
         data = b"\x01\x02\x03\x04\x05\x06\x07\x08"
         for word_size in [8, 4]:
@@ -1047,6 +1096,7 @@ class TestCoreDump(TestCase):
     def test_simple(self):
         data = b"hello, world"
         prog = Program()
+        self.assertIsNone(prog.core_dump_path)
         with tempfile.NamedTemporaryFile() as f:
             f.write(
                 create_elf_file(
@@ -1055,6 +1105,7 @@ class TestCoreDump(TestCase):
             )
             f.flush()
             prog.set_core_dump(f.name)
+        self.assertEqual(prog.core_dump_path, f.name)
         self.assertEqual(prog.read(0xFFFF0000, len(data)), data)
         self.assertRaises(FaultError, prog.read, 0x0, len(data), physical=True)
 
