@@ -2,18 +2,19 @@
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
 
-from collections import OrderedDict
+import dataclasses
 import inspect
 import os
 from pathlib import Path
 from typing import Dict, Mapping, NamedTuple, Sequence
 
 from _drgn_util.platform import NORMALIZED_MACHINE_NAME
-from util import KernelVersion
 
 # Kernel versions that we run tests on and therefore support. Keep this in sync
 # with docs/support_matrix.rst.
 SUPPORTED_KERNEL_VERSIONS = (
+    "6.18",
+    "6.17",
     "6.16",
     "6.15",
     "6.14",
@@ -142,6 +143,8 @@ CONFIG_XFS_FS=m
 
 # For mm tests.
 CONFIG_HUGETLBFS=y
+CONFIG_MEMORY_HOTPLUG=y
+CONFIG_MEMORY_HOTREMOVE=y
 
 # For net tests.
 CONFIG_NAMESPACES=y
@@ -149,6 +152,9 @@ CONFIG_NAMESPACES=y
 # For nodemask tests.
 CONFIG_NUMA=y
 
+# For sched tests.
+CONFIG_SCHEDSTATS=y
+
 # For slab allocator tests.
 CONFIG_SLAB_FREELIST_HARDENED=y
 
@@ -183,14 +189,15 @@ CONFIG_USER_NS=y
 """
 
 
-class KernelFlavor(NamedTuple):
+@dataclasses.dataclass(frozen=True, eq=False)
+class KernelFlavor:
     name: str
     description: str
     config: str
 
 
-KERNEL_FLAVORS = OrderedDict(
-    (flavor.name, flavor)
+KERNEL_FLAVORS = {
+    flavor.name: flavor
     for flavor in (
         KernelFlavor(
             name="default",
@@ -213,6 +220,10 @@ KERNEL_FLAVORS = OrderedDict(
                 # CONFIG_SLAB to CONFIG_SLAB_DEPRECATED") (in v6.5) renamed the
                 # option for SLAB.
                 CONFIG_SLAB_DEPRECATED=y
+                # Linux kernel commit 16a1d968358a ("mm/slab: remove mm/slab.c
+                # and slab_def.h") (in v6.8) removed SLAB. Test this
+                # non-default SLUB option instead.
+                CONFIG_SLUB_CPU_PARTIAL=n
                 CONFIG_MODVERSIONS=y
                 CONFIG_RANDOMIZE_BASE=n
             """,
@@ -238,10 +249,11 @@ KERNEL_FLAVORS = OrderedDict(
             """,
         ),
     )
-)
+}
 
 
-class Architecture(NamedTuple):
+@dataclasses.dataclass(frozen=True, eq=False)
+class Architecture:
     # Architecture name. This matches the names used by
     # _drgn_util.platform.NORMALIZED_MACHINE_NAME and qemu-system-$arch_name.
     name: str
@@ -386,7 +398,15 @@ ARCHITECTURES = {
                 CONFIG_SERIAL_8250=y
                 CONFIG_SERIAL_8250_CONSOLE=y
             """,
-            kernel_flavor_configs={},
+            kernel_flavor_configs={
+                "alternative": """
+                    CONFIG_UNWINDER_FRAME_POINTER=y
+                    # Before Linux kernel commit 11af847446ed ("x86/unwind:
+                    # Rename unwinder config options to 'CONFIG_UNWINDER_*'")
+                    # (in v4.15).
+                    CONFIG_FRAME_POINTER_UNWINDER=y
+                """,
+            },
             kernel_org_compiler_name="x86_64-linux",
             qemu_options=("-nodefaults",),
             qemu_console="ttyS0",
@@ -433,25 +453,13 @@ def kconfig_localversion(arch: Architecture, flavor: KernelFlavor, version: str)
     vmtest_kernel_version = [
         # Increment the major version to rebuild every
         # architecture/flavor/version combination.
-        35,
+        37,
         # The minor version makes the default flavor the "latest" version.
         1 if flavor.name == "default" else 0,
     ]
     patch_level = 0
     # If only specific architecture/flavor/version combinations need to be
     # rebuilt, conditionally increment the patch level here.
-    kver = KernelVersion(version)
-    if KernelVersion("4.18") <= kver < KernelVersion("5.18"):
-        patch_level += 1
-    if (
-        (KernelVersion("6.6") <= kver < KernelVersion("6.6.3"))
-        or (KernelVersion("6.2") <= kver < KernelVersion("6.5.13"))
-        or (KernelVersion("5.16") <= kver < KernelVersion("6.1.64"))
-        or (KernelVersion("5.11") <= kver < KernelVersion("5.15.140"))
-        or (KernelVersion("5.5") <= kver < KernelVersion("5.10.202"))
-        or (kver < KernelVersion("5.4.262"))
-    ):
-        patch_level += 1
     if patch_level:
         vmtest_kernel_version.append(patch_level)
 
