@@ -26,12 +26,11 @@ from vmtest.config import (
     HOST_ARCHITECTURE,
     KERNEL_FLAVORS,
     Architecture,
-    Compiler,
     KernelFlavor,
     kconfig,
     kconfig_localversion,
 )
-from vmtest.download import COMPILER_URL, DownloadCompiler, download
+from vmtest.download import COMPILER_URL, Downloader
 
 logger = logging.getLogger(__name__)
 
@@ -180,6 +179,44 @@ _PATCHES = (
             (None, KernelVersion("5.4.262")),
         ),
     ),
+    _Patch(
+        name="kbuild-Only-add-fno-var-tracking-assignments-for-old.patch",
+        versions=((KernelVersion("5.1"), KernelVersion("5.10")),),
+    ),
+    _Patch(
+        name="4.19-kbuild-Only-add-fno-var-tracking-assignments-for-old.patch",
+        versions=((None, KernelVersion("5.1")),),
+    ),
+    # For aarch64, some IRQ stacks are mapped in the range [_text, _stext],
+    # which does not get mapped in /proc/vmcore. This patch is in linux-next for
+    # 6.18, and it is needed since e2a073dde9218 ("arm64: omit [_text, _stext)
+    # from permanent kernel mapping") in v5.11.
+    _Patch(
+        name="6.17-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
+        versions=((KernelVersion("6.9"), KernelVersion("6.18")),),
+    ),
+    _Patch(
+        name="6.8-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
+        versions=(
+            (KernelVersion("6.1"), KernelVersion("6.2")),
+            (KernelVersion("6.4"), KernelVersion("6.9")),
+        ),
+    ),
+    _Patch(
+        name="6.3-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
+        versions=(
+            (KernelVersion("5.14"), KernelVersion("6.1")),
+            (KernelVersion("6.2"), KernelVersion("6.4")),
+        ),
+    ),
+    _Patch(
+        name="5.13-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
+        versions=((KernelVersion("5.13"), KernelVersion("5.14")),),
+    ),
+    _Patch(
+        name="5.12-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
+        versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
+    ),
 )
 
 
@@ -209,6 +246,7 @@ async def apply_patches(kernel_dir: Path) -> None:
             cwd=kernel_dir,
             stderr=asyncio.subprocess.PIPE,
         )
+        assert proc.stderr is not None  # for mypy
         stderr = await proc.stderr.read()
         if await proc.wait() != 0:
             try:
@@ -494,6 +532,7 @@ static int __init test_init(void)
 module_init(test_init);
 
 MODULE_LICENSE("GPL");
+MODULE_DESCRIPTION("Module for testing build");
 """
             )
             (test_module_dir / "Makefile").write_text("obj-m := test.o\n")
@@ -514,9 +553,15 @@ MODULE_LICENSE("GPL");
                 stderr=asyncio.subprocess.PIPE,
                 env=self._env,
             )
+            assert proc.stdout is not None  # for mypy
+            assert proc.stderr is not None  # for mypy
             try:
-                stdout_task = asyncio.create_task(proc.stdout.readline())
-                stderr_task = asyncio.create_task(proc.stderr.readline())
+                stdout_task: Optional[asyncio.Task[bytes]] = asyncio.create_task(
+                    proc.stdout.readline()
+                )
+                stderr_task: Optional[asyncio.Task[bytes]] = asyncio.create_task(
+                    proc.stderr.readline()
+                )
                 error = False
                 while stdout_task is not None or stderr_task is not None:
                     aws = []
@@ -771,8 +816,8 @@ async def main() -> None:
     if hasattr(args, "download_compiler"):
         if args.download_compiler is None:
             args.download_compiler = default_download_compiler_directory
-        downloaded = next(download(args.download_compiler, [DownloadCompiler(arch)]))
-        assert isinstance(downloaded, Compiler)
+        downloader = Downloader(args.download_compiler)
+        downloaded = downloader.download_compiler(downloader.resolve_compiler(arch))
         env = {**os.environ, **downloaded.env()}
     else:
         env = None
