@@ -12,14 +12,14 @@ import socket
 import subprocess
 import sys
 import tempfile
-from typing import Any, Optional, Sequence
+from typing import Any, Optional, Sequence, TextIO
 
 from util import nproc, out_of_date
 from vmtest.config import HOST_ARCHITECTURE, Kernel, local_kernel
 from vmtest.download import (
     DOWNLOAD_KERNEL_ARGPARSE_METAVAR,
+    Downloader,
     DownloadKernel,
-    download,
     download_kernel_argparse_type,
 )
 from vmtest.kmod import build_kmod
@@ -224,8 +224,10 @@ def run_in_vm(
     build_dir: Path,
     *,
     extra_qemu_options: Sequence[str] = (),
+    extra_kernel_cmdline: Sequence[str] = (),
     test_kmod: TestKmodMode = TestKmodMode.NONE,
     interactive: bool = False,
+    outfile: Optional[TextIO] = None,
 ) -> int:
     if root_dir is None:
         if kernel.arch is HOST_ARCHITECTURE:
@@ -234,7 +236,7 @@ def run_in_vm(
             root_dir = build_dir / kernel.arch.name / "rootfs"
 
     if test_kmod != TestKmodMode.NONE:
-        kmod = build_kmod(build_dir, kernel)
+        kmod = build_kmod(build_dir, kernel, outfile=outfile)
 
     qemu_exe = "qemu-system-" + kernel.arch.name
     match = re.search(
@@ -352,6 +354,17 @@ def run_in_vm(
         with disk_path.open("wb") as f:
             os.ftruncate(f.fileno(), 1024 * 1024 * 1024)
 
+        kernel_cmdline = [
+            "rootfstype=9p",
+            f"rootflags={_9pfs_mount_options}",
+            "ro",
+            f"console={kernel.arch.qemu_console},115200",
+            "panic=-1",
+            "crashkernel=256M",
+            *extra_kernel_cmdline,
+            f"init={init}",
+        ]
+
         proc = subprocess.Popen(
             [
                 # fmt: off
@@ -360,8 +373,9 @@ def run_in_vm(
 
                 qemu_exe, *kvm_args,
 
-                # Limit the number of cores to 8, otherwise we can reach an OOM troubles.
-                "-smp", str(min(nproc(), 8)), "-m", "2G",
+                # Limit the number of cores to 2. We want to test SMP, but each additional
+                # virtualized CPU costs memory and CPU time, so 2 is enaugh.
+                "-smp", str(min(nproc(), 2)), "-m", "2G",
 
                 "-display", "none", *serial_args,
 
@@ -385,13 +399,14 @@ def run_in_vm(
                 *kernel.arch.qemu_options,
 
                 "-kernel", str(kernel.path / "vmlinuz"),
-                "-append",
-                f"rootfstype=9p rootflags={_9pfs_mount_options} ro console={kernel.arch.qemu_console},115200 panic=-1 crashkernel=256M init={init}",
+                "-append", " ".join(kernel_cmdline),
 
                 *extra_qemu_options,
                 # fmt: on
             ],
             env=env,
+            stdout=outfile,
+            stderr=outfile,
             stdin=infile,
         )
         try:
@@ -493,6 +508,13 @@ if __name__ == "__main__":
         default=argparse.SUPPRESS,
         help="additional option to pass to QEMU (not split on spaces). May be given multiple times",
     )
+    parser.add_argument(
+        "--kernel-cmdline",
+        metavar="PARAMETER",
+        action="append",
+        default=argparse.SUPPRESS,
+        help="additional parameters to pass on kernel command line. May be given multiple times",
+    )
     parser.add_argument(
         "--build-test-kmod",
         dest="test_kmod",
@@ -524,13 +546,21 @@ if __name__ == "__main__":
         args.root_directory = None
     if not hasattr(args, "qemu_options"):
         args.qemu_options = []
+    if not hasattr(args, "kernel_cmdline"):
+        args.kernel_cmdline = []
     if not hasattr(args, "test_kmod"):
         args.test_kmod = TestKmodMode.NONE
 
+    downloader = Downloader(args.directory)
+    if args.test_kmod != TestKmodMode.NONE:
+        downloader.download_compiler(downloader.resolve_compiler(args.kernel.arch))
+
     if args.kernel.pattern.startswith(".") or args.kernel.pattern.startswith("/"):
         kernel = local_kernel(args.kernel.arch, Path(args.kernel.pattern))
     else:
-        kernel = next(download(args.directory, [args.kernel]))  # type: ignore[assignment]
+        kernel = downloader.download_kernel(
+            downloader.resolve_kernel(args.kernel.arch, args.kernel.pattern)
+        )
 
     try:
         command = (
@@ -545,6 +575,7 @@ if __name__ == "__main__":
                 args.root_directory,
                 args.directory,
                 extra_qemu_options=args.qemu_options,
+                extra_kernel_cmdline=args.kernel_cmdline,
                 test_kmod=args.test_kmod,
                 interactive=True,
             )
